/** 
 * FigureDrawer allows the user to interactively 
 * draw an ellipse by pressing the mouse on the 
 * center point and dragging the mouse to specify 
 * the radius. It sets transparent colors based on 
 * the angle of the radius.
 *
 * @author kvlinden, snelesen
 * @version Fall, 2010
 */

final int WIDTH = 500, HEIGHT = WIDTH;
int figureX, figureY;
float diameter;  
int eraseLine;              // position of erase line

void setup() {
  size(WIDTH, HEIGHT);
  background(255);
  smooth();
  colorMode(HSB, 2 * PI, 100, 100);
  eraseLine = -2;
}

// There must be a draw method.
void draw() {
}

/**
 * Allow the user to draw an ellipse of arbitrary size 
 * by pressing the mouse at the center point and dragging
 * the mouse to specify the radius.
 */
void mousePressed() {
  figureX = mouseX;
  figureY = mouseY;
  diameter = 0;
  stroke(0);
  strokeWeight(1);
}
void mouseDragged() { 
  diameter = 2 * computeDistance(figureX, figureY, mouseX, mouseY);
  fill(computeColor(figureX, figureY, mouseX, mouseY));
  drawCircle(figureX, figureY, diameter);
}

/**
 * Draw circle is a simple abstraction that draws circles as ellipses 
 * with equal width and height.
 */
void drawCircle(int x, int y, float diameter) {
  ellipse(x, y, diameter, diameter);
}

/**
 * Press and hold any key to erase the board with a 
 * vertical line going from left to right (cf. Etch-a-sketch).
 */
void keyPressed() {
  save("figure.png");
  eraseLine += 2;
  stroke(TWO_PI);
  strokeWeight(4);
  line(eraseLine, 0, eraseLine, height);
}
void keyReleased() {
  eraseLine = -1;
}

/**
 * Compute the Euclidean distance from (x1, y1) to (x2, y2).
 * (This partially reimplements Processing's dist() method).
 *
 * @param x1 the x coordinate of position 1
 * @param y1 the y coordinate of position 1
 * @param x2 the x coordinate of position 2
 * @param y2 the y coordinate of position 2
 * @return the distance from (x1, y1) to (x2, y2)
 */
float computeDistance(int x1, int y1, int x2, int y2) {
  float dx = x1-x2;
  float dy = y1-y2;
  return sqrt(dx*dx + dy*dy);
}

/**
 * computeColor returns a color object based on the 
 * figure's specifications.
 *
 * @param sourceX the source's x coordinate
 * @param sourceY the source's y coordinate
 * @param targetX the target's x coordinate
 * @param targetY the target's y coordinate
 * @return a semi-transparent color based on the parameters
 */
color computeColor(int sourceX, int sourceY, int targetX, int targetY) {
  return color(PI + computeFacingAngle(sourceX, sourceY, targetX, targetY), 100, 100, 10);
}

/**
 * computeFacingAngle returns the angle, in radians, between 
 * (sourceX, sourceY) and (targetX, targetY).
 *
 * @param sourceX the source x coordinate
 * @param sourceY the source y coordinate
 * @param targetX the target x coordinate
 * @param targetY the target y coordinate
 * @return angle from source to target in radians
 */
float computeFacingAngle(float sourceX, float sourceY, float targetX, float targetY) {
  return atan2(targetY - sourceY, targetX - sourceX);
}

// This mouse movement stuff is interesting, but too idiosyncratic for the lecture.
void mouseMoved() {
  // Erase the figure from where it was before.
  stroke(255);
  strokeWeight(2);
  ellipse(figureX, figureY, diameter, diameter);
  
  // Move the figure based on mouse movement.
  figureX += mouseX - pmouseX;
  figureY += mouseY - pmouseY;
  stroke(0);
  strokeWeight(1);
  ellipse(figureX, figureY, diameter, diameter);
}

