package c07repetition.lecture.mandelbrot;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * CS108 Final Project
 * This is a simple gui interface that encapsulates the Mandelbrot processing sketch.
 * 
 * @author gao2
 * @version Fall, 2009
 * @author modified by kvlinden
 */
public class MandelbrotController extends JFrame implements ChangeListener, ActionListener {

	public MandelbrotPanel mandelbrot;	
	public JLabel messageLabel;
	public JButton blackButton, colorButton;

	public MandelbrotController() {
		super("Mandelbrot");
		setLayout(new BorderLayout());
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		// add processing panel
		mandelbrot = new MandelbrotPanel();
		mandelbrot.init();
		add(mandelbrot, BorderLayout.NORTH);
		// add an iteration slider
		JSlider iterations = new JSlider(JSlider.HORIZONTAL, 0, 40, 40);
		iterations.setMajorTickSpacing(10);
		iterations.setPaintTicks(true);
		iterations.setPaintLabels(true);
		iterations.addChangeListener(this);
		add(iterations, BorderLayout.CENTER);
		// add a color changer multiplier box thing and black/white button
		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout());
		blackButton = new JButton("gray");
		blackButton.addActionListener(this);
		controlPanel.add(blackButton);
		colorButton = new JButton("color");
		colorButton.addActionListener(this);
		colorButton.setEnabled(false);
		controlPanel.add(colorButton);
		JLabel colorChangerLabel = new JLabel("Color Factor");
		controlPanel.add(colorChangerLabel);
		JTextField colorFactor = new JTextField("1", 2);
		colorFactor.addActionListener(this);
		controlPanel.add(colorFactor);
		messageLabel = new JLabel("                                     ");
		controlPanel.add(messageLabel);
		add(controlPanel, BorderLayout.SOUTH);

	}

	// deal with the changing sliders
	public void stateChanged(ChangeEvent e) {
		JSlider slider = (JSlider) e.getSource();
		int value = slider.getValue();
		try {
			mandelbrot.setIterations(value);
		} catch (Exception e1) {
			messageLabel.setText("wrong!" + e1.getMessage());
			System.out.println(e1);
		}

	}

	public void actionPerformed(ActionEvent e) {
		// gray button grays out color button
		if (e.getActionCommand().equalsIgnoreCase("gray")) {
			MandelbrotPanel.blackAndWhite();
			blackButton.setEnabled(false);
			colorButton.setEnabled(true);
			// color button grays out gray button
		} else if (e.getActionCommand().equalsIgnoreCase("color")) {
			MandelbrotPanel.setToColor();
			blackButton.setEnabled(true);
			colorButton.setEnabled(false);
			// this is the action event for the slider
		} else {
			try {
				JTextField field = (JTextField) e.getSource();
				String value = field.getText();
				mandelbrot.setColorChanger(Integer.parseInt(value));

			} catch (Exception exception) {
				// this puts the errors up on the gui
				messageLabel.setText("Illegal format: " + exception.getMessage());
				System.out.println(exception);
			}
		}
	}

	public static void main(String[] args) {
		MandelbrotController application = new MandelbrotController();
		application.pack();
		application.setVisible(true);
	}

}
