package c07repetition.lecture.mandelbrot;

import processing.core.PApplet;

/**
 * CS108 Final Project 
 * 
 * Mandelbrot algorithm taken from Recursive Structures and Processes by Curran
 * Kelleher August 24, 2007
 * 
 * @author gao2
 * @version Fall, 2009
 * @author modified by kvlinden
 */

public class MandelbrotPanel extends PApplet {

	private int maxIterations;
	private static int COLOR_CHANGER;
	private static int redFactor, greenFactor, blueFactor;
	Complex z = new Complex(-2.2, 2.2);
	Complex c = new Complex(-2.2, 2.2);

	public MandelbrotPanel() {
		maxIterations = 40;
		COLOR_CHANGER = 1;
		redFactor = 10;
		greenFactor = 12;
		blueFactor = 17;
	}

	public MandelbrotPanel(int iterations) {
		maxIterations = iterations;
		COLOR_CHANGER = 1;
	}

	public void setIterations(int iterations) throws Exception {
		maxIterations = iterations;
	}

	public int getIterations() {
		return maxIterations;
	}

	// mutator method to set color changer
	public void setColorChanger(int colorChanger) throws Exception {
		COLOR_CHANGER = colorChanger;
	}

	// accessor method to get color changer value
	public int getColorChanger() {
		return COLOR_CHANGER;
	}

	// mutator method to change things to black and white
	public static void blackAndWhite() {
		redFactor = 1;
		greenFactor = 1;
		blueFactor = 1;
	}

	// mutator method to make things in color again
	public static void setToColor() {
		redFactor = 10;
		greenFactor = 12;
		blueFactor = 17;
	}

	// accessor methods to get the color factors
	public int getRedFactor() {
		return redFactor;
	}

	public int getGreenFactor() {
		return greenFactor;
	}

	public int getBlueFactor() {
		return blueFactor;
	}

	// make a nice big canvas
	public void setup() {
		size(600, 600);
	}

	// this draw the mandelbrot set
	public void draw() {
		for (int xPixel = 0; xPixel < width; xPixel++)
			for (int yPixel = 0; yPixel < height; yPixel++) {
				c.setReal(map((float) xPixel, (float) 0, (float) width, (float) -2.2, (float) 2.2));
				c.setImag(map((float) yPixel, (float) 0, (float) width, (float) -2.2, (float) 2.2));
				int iterations = calculatIterations(c);
				int d = calculateColor(iterations);
				set(xPixel, yPixel, d);
			}
	}

	// this keeps track of how many iterations of the equation
	// so that the program knows what color to color each pixel
	private int calculatIterations(Complex c) {
		z.setReal(0);
		z.setImag(0);
		int iterations = 0;
		while (circleContains(z) && iterations < maxIterations) {
			z = z.times(z).plus(c);
			iterations++;
		}
		return iterations;
	}

	private boolean circleContains(Complex z2) {
		return Math.sqrt(z.real() * z.real() * z.imag() * z.imag()) < 2;
	}

	// this calculates which color
	// the if redFactor, greenFactor, and blueFactor == 1,
	// the image turns grayscale
	private int calculateColor(int iterations) {
		if (iterations == maxIterations) {
			return color(0);
		} else {
			int myRed = (iterations * (COLOR_CHANGER * redFactor)) % 255;
			int myGreen = (int) ((iterations * (COLOR_CHANGER * greenFactor)) % 255);
			int myBlue = (int) ((iterations * (COLOR_CHANGER * blueFactor)) % 255);
			return color(myRed, myGreen, myBlue);

		}
	}

}
