package c07repetition.lecture.treeGUI;

import processing.core.PApplet;

/**
 * TreePanel encapsulates the Tree Processing sketch, which draws a tree fractal using recursion.
 * (reference: Curran Kelleher, "Recursive Structures and Processes", MIT, 2007)
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
public class TreePanel extends PApplet {

	private static final int RECURSION_LEVEL = 10, TRUNK_LEVEL = RECURSION_LEVEL / 2;
	private float myBranchAngle, myNoiseFactor;

	public TreePanel(int branchAngle) {
		myBranchAngle = radians(branchAngle);
		myNoiseFactor = 0.0f;
	}

	public void setBranchAngle(double branchAngleDegrees) {
		myBranchAngle = radians((float) branchAngleDegrees);
	}

	public float getBranchAngle() {
		return myBranchAngle;
	}

	public void setNoiseFactor(double noiseFactor) {
		myNoiseFactor = (float) noiseFactor;
	}

	public float getNoiseFactor() {
		return myNoiseFactor;
	}

	public void setup() {
		size(500, 500);
		frameRate(5);
		smooth();
	}

	public void draw() {
		background(255);
		drawTree(width / 2, height, height * 0.4f, PI / 2, RECURSION_LEVEL);
	}

	private void drawTree(float x1, float y1, float length, float angle, int depth) {
		float x2 = x1 - cos(angle) * length;
		float y2 = y1 - sin(angle) * length;
		if (depth < TRUNK_LEVEL) {
			stroke(0, 200, 0);
			strokeWeight(1);
		} else {
			stroke(95, 75, 50);
			strokeWeight(depth - TRUNK_LEVEL);
		}
		line(x1, y1, x2, y2);
		if (depth > 0) {
			drawTree(x2, y2, length * 0.592f + random(-myNoiseFactor, myNoiseFactor), angle + myBranchAngle / 2
					+ random(-myNoiseFactor, myNoiseFactor), depth - 1);
			drawTree(x2, y2, length * 0.592f + random(-myNoiseFactor, myNoiseFactor), angle - myBranchAngle / 2
					+ random(-myNoiseFactor, myNoiseFactor), depth - 1);
		}
	}

}
