package c07repetition.text.functionAnimator;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

@SuppressWarnings("serial")
public class FunctionAnimatorController extends JFrame implements ActionListener, ChangeListener {

	private final Double INITIAL_SLOPE = 1.0, INITIAL_Y_INTERCEPT = 0.1;

	private FunctionAnimatorPanel functionAnimator;
//	private JPanel functionAnimator;

	private JTextField slopeField, yInterceptField;
	@SuppressWarnings("unused")
	private JSlider slopeSlider, yInterceptSlider;

	public FunctionAnimatorController() {
		setTitle("Function Animator");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		functionAnimator = new FunctionAnimatorPanel(INITIAL_SLOPE.floatValue(), INITIAL_Y_INTERCEPT.floatValue());
		functionAnimator.init();
//		functionAnimator = new JPanel();
		functionAnimator.setPreferredSize(new Dimension(300, 300));
		functionAnimator.setBackground(new Color(150, 150, 150));
		add(functionAnimator, BorderLayout.NORTH);

		JPanel controller = new JPanel();

		// The text-field interface
		 controller.setLayout(new FlowLayout());
		 controller.add(new JLabel("slope:"));
		 slopeField = new JTextField(INITIAL_SLOPE.toString());
		 slopeField.setFont(new Font("Ariel", 0, 28));
		 slopeField.addActionListener(this);
		 slopeField.setActionCommand("slope");
		 controller.add(slopeField);
		 controller.add(new JLabel("yIntercept:"));
		 yInterceptField = new JTextField(INITIAL_Y_INTERCEPT.toString());
		 yInterceptField.setFont(new Font("Ariel", 0, 28));
		 yInterceptField.addActionListener(this);
		 yInterceptField.setActionCommand("yIntercept");
		 controller.add(yInterceptField);

		// The slider interface
//		controller.setLayout(new BorderLayout());
//		controller.add(createSlideControl("slope"), BorderLayout.NORTH);
//		controller.add(createSlideControl("yIntercept"), BorderLayout.CENTER);

		add(controller, BorderLayout.SOUTH);
	}

	@SuppressWarnings("unused")
	private JPanel createSlideControl(String name) {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		panel.add(new JLabel(name + ": "));
		JSlider slider = new JSlider(JSlider.HORIZONTAL, -10, 10, 1);
		slider.setName(name);
		slider.setMajorTickSpacing(5);
		slider.setMinorTickSpacing(1);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		slider.addChangeListener(this);
		panel.add(slider);
		return panel;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("slope")) {
			double slope = Double.parseDouble(slopeField.getText());
			functionAnimator.setSlope(slope);
		} else if (command.equals("yIntercept")) {
			double yIntercept = Double.parseDouble(yInterceptField.getText());
			functionAnimator.setYIntercept(yIntercept);
		}
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		JSlider slider = (JSlider) e.getSource();
		double value = slider.getValue() / 10.0;
		if (slider.getName().equals("slope")) {
			functionAnimator.setSlope(value);
		} else if (slider.getName().equals("yIntercept")) {
			functionAnimator.setYIntercept(value);
		}
	}

	public static void main(String[] args) {
		FunctionAnimatorController application = new FunctionAnimatorController();
		application.pack();
		application.setVisible(true);
		application.setLocation(100, 100);
	}

}
