package c07repetition.text.functionAnimator;

import processing.core.PApplet;

@SuppressWarnings("serial")
public class FunctionAnimatorPanel extends PApplet {
	
	private final int SIZE = 300;

	private float mySlope, myYIntercept;
	private int myWidth, myHeight, myXAxis, myYAxis;
	private float myXMax, myScale, myXMin, myYMax, myYMin;

	public FunctionAnimatorPanel(float slope, float yIntercept) {
		mySlope = slope;
		myYIntercept = yIntercept;
		myWidth = myHeight = SIZE;
		myXAxis = myHeight/2;
		myYAxis = myWidth/2;
		myXMax = 1;
		myScale = (myWidth - myYAxis) / myXMax;
		myXMin = -1 * (myWidth - myYAxis) / myScale;
		myYMax = (myHeight - myXAxis) / myScale;
		myYMin = -1 * (myHeight - myXAxis) / myScale;
	}
	
	public void setSlope(double slope) {
		mySlope = (float)slope;
	}
	
	public void setYIntercept(double yIntercept) {
		myYIntercept = (float)yIntercept;
	}

	public void setup() {
		size(myWidth, myHeight);
		smooth();
	}

	public void draw() {
		background(255);
		drawAxes(myXAxis, myYAxis);
		plotFunction();
	}

	private void drawAxes(int x, int y) {
		stroke(200);
		line(0, myXAxis, myWidth, myXAxis); // x axis
		line(myYAxis, 0, myYAxis, myHeight); // y axis
	}

	private void plotFunction() {
		stroke(0);
		for (float x = myXMin; x <= myXMax; x += 0.0001) {
			point(translateX(x), translateY(computeY(x)));
		}
	}

	private float translateX(float value) {
		return value * myScale + myYAxis;
	}

	private float translateY(float value) {
		return value * -1 * myScale + myXAxis;
	}

	private float computeY(float x) {
		return mySlope * x + myYIntercept;
	}

}
