/** 
 * Mandelbrot draws a Mandelbrot set using a while statement (among other things)
 * (reference: Curran Kelleher, "Recursive Structures and Processes", MIT, 2007)
 *
 * The user can re-center the image with a mouse click and zoom in/out with +/-.
 *
 * @author kvlinden
 * @version Fall, 2009
 */

final int SIZE = 300, ITERATIONS = 100;
final double ZOOM_FACTOR = 0.10;
Complex c, z;
double xBottom, yBottom, xTop, yTop;

void setup() {
  size(SIZE, SIZE);
  xBottom = yBottom = -2.2;
  xTop = yTop = 2.2;
  drawMandelbrot();
}

void draw() {
}

void drawMandelbrot() {
  for (int xPixel = 0; xPixel < width; xPixel++) {
    for (int yPixel = 0; yPixel < height; yPixel++) {
      c = new Complex(map((float)xPixel, (float)0, (float)width, (float)xBottom, (float)xTop), map((float)yPixel, (float)0, (float)height, (float)yBottom, (float)yTop));
      int iterations = calculateIterations(c);
      color c = calculateColor(iterations);
      stroke(c);
      point(xPixel, yPixel);
    }
  }
}

int calculateIterations(Complex c) {
  z = new Complex(0, 0);
  int iterations = 0;
  while (circleContains(z) && iterations < ITERATIONS) {
    z = z.times(z).plus(c);
    iterations++;
  }
  return iterations;
}

boolean circleContains(Complex z) {
  return Math.sqrt((z.real() * z.real() + z.imag() * z.imag())) < 2;
}

color calculateColor(int iterations) {
  if (iterations == ITERATIONS) {
    return color(0);
  } 
  else {
    return color((iterations * 10) % 255, (iterations * 12) % 255, (iterations * 17) % 255);
  }
}

void mousePressed() {
  double xDelta = mapDouble(mouseX-width/2, -width/2, width/2, -(xTop-xBottom)/2, (xTop-xBottom)/2);
  double yDelta = mapDouble(mouseY-height/2, -height/2, height/2, -(yTop-yBottom)/2, (yTop-yBottom)/2);
  xTop += xDelta;
  xBottom += xDelta;
  yTop += yDelta;
  yBottom += yDelta;
  drawMandelbrot();
}

void keyPressed() {
  if (key == 'p') {
    save("mandelbrot.tif");
  }
  if ((key != '+') && (key != '-')) {
    return;
  }
  double delta = (xTop - xBottom) * ZOOM_FACTOR;
  if (key=='-') {
    delta *= -1;
  }
  xTop -= delta;
  yTop -= delta;
  xBottom += delta;
  yBottom += delta;
  drawMandelbrot();
}

double mapDouble(double value, double startLow, double startHigh, double endLow, double endHigh) {
  return endLow + ((value - startLow) / (startHigh - startLow)) * (endHigh - endLow);
}







