/**
 * CentralAmericaPopulationMap displays the population of the seven 
 * Central American states using data from Wikipedia.
 *
 * data: http://en.wikipedia.org/wiki/Central_america
 * world image: http://commons.wikimedia.org/wiki/File:BlankMap-World6.svg
 * country codes: see http://www.iso.org/iso/iso_3166_code_lists
 *
 * @author: kvlinden
 * @version Fall, 2009
 */

PShape world;
String[] populationLines, populationCountries;
float[] populationValues;
float maximumPopulation , minimumPopulation ;

void setup() {
  size(800, 600);
  world = loadShape("BlankMap-World6.svg");
  loadPopulationData("populationCAISO.txt");
  smooth();
  noLoop();
}

void draw() {
  background(255);
  translate(-2100, -2250);
  scale(4.0);
  shape(world);
  float intensityValue;
  PShape country;
  for (int i = 0; i < populationCountries.length; i++) {
    country = world.getChild(populationCountries[i].toLowerCase());
    if (country != null) {
      country.disableStyle();
      intensityValue = map(populationValues[i], minimumPopulation , maximumPopulation, 
                           0, 255);
      fill(255 - intensityValue, 255 - intensityValue, 255);
      //fill(random(255), random(255), random(255));
      shape(country);
    }
  }
}

void loadPopulationData(String filename) {
  String[] aLine;
  populationLines = loadStrings(filename);
  populationCountries = new String[populationLines.length];
  populationValues = new float[populationLines.length];
  maximumPopulation  = Integer.MIN_VALUE;
  minimumPopulation  = Integer.MAX_VALUE;
  for (int i = 0; i < populationLines.length; i++) {
    aLine = split(populationLines[i], " ");
    populationCountries[i] = aLine[0];
    populationValues[i] = float(aLine[1]);
    if (populationValues[i] > maximumPopulation ) {
      maximumPopulation  = populationValues[i];
    }
    if (populationValues[i] < minimumPopulation ) {
      minimumPopulation  = populationValues[i];
    }
  }
}

void keyPressed() {
  save("../../../../web/c06arrays/centralAmerica.png");
}












