/**
 * USStimulusMap implements a map of United States stimulus spending 
 * data with file input (cf. http://www.recovery.gov/?q=content/investments-state)
 *
 * @author kvlinden
 * @version Fall, 2009
 */

PShape usa, state;

String[] states, aLine, stimulusStrings;
int[] values;
int minValue, maxValue;

void setup() {
  stimulusStrings = loadStrings("Stimulus_By_State_2009.txt");
  states = new String[stimulusStrings.length];
  values = new int[stimulusStrings.length];
  minValue = MAX_INT;
  maxValue = -1;
  for (int i = 0; i < stimulusStrings.length; i++) {
    aLine = split(stimulusStrings[i], " ");
    states[i] = aLine[0];
    values[i] = int(aLine[1]);
    if (values[i] < minValue) {
      minValue = values[i];
    }
    if (values[i] > maxValue) {
      maxValue = values[i];
    }
  }
  size(950, 600);
  usa = loadShape("Blank_US_Map.svg");
  smooth();
  noLoop();
}

void draw() {
  background(255);
  shape(usa, 0, 0);
  for (int i = 0; i < states.length; i++) {
    state = usa.getChild(states[i]);
    state.disableStyle();
    fill(color(0, 0, 255, map(values[i], minValue, maxValue, 10, 255)));
    noStroke();
    shape(state, 0, 0);
  }  
}

void keyPressed() {
  save("chart.tif");
}



