package c09classes.lecture.icecream;


/**
 * IceCreamOrder models an order for an ice cream shop
 * 
 * IceCreamOrder invariant
 * (myScoops >= 1) 
 * @author smn4
 * @version Spring 2012
 */
public class IceCreamOrder {

	//Class data
	private static final double PRICE_PER_SCOOP = 1.50;

	//Instance data
	private int myScoops;
	private String myFlavor;
	private boolean myStatus;


	/**
	 * Construct a new IceCreamOrder with default values
	 */	
	public IceCreamOrder(){
		myScoops  = 1;
		myFlavor = "Vanilla";
		myStatus = false;
	}

	/**  
	 * Construct a new IceCreamOrder
	 * @param scoops the number of scoops
	 * @param flavor the ice cream flavor
	 * @param status the order status
	 */
	public IceCreamOrder(int scoops, String flavor, boolean status) throws Exception {
		if(isValidScoops (scoops)){
			myScoops = scoops;
		}
		else{
			//System.err.println("Invalid number of scoops: " + scoops);
			//System.exit(-1);
			throw new Exception("Invalid number of scoops: " + scoops);
		}
		myFlavor = flavor;
		myStatus = status;
	}


	/**
	 * Indicate whether or not the number of scoops is valid.
	 * @param scoops the value to check
	 * @return true if valid, false otherwise
	 */
	private boolean isValidScoops(int scoops) {
		return scoops >= 1;
	}


	/**
	 * Print out the value of the order
	 */
	public String toString() {
		String result = myScoops + " scoop(s) of " + myFlavor + " :";
		if (myStatus){
			result += " Fulfilled";
		}
		else{
			result += " Unfulfilled";
		}
		return result;
	}

	//Accessors for scoops and flavor
	public int getScoops(){
		return myScoops;
	}

	public String getFlavor(){
		return myFlavor;
	}


	/**
	 * Set a new number of scoops for the order
	 * @param scoops non-negative scoops value
	 */
	public void setScoops(int scoops) {
		if (isValidScoops(scoops)){
			myScoops = scoops;
		}
		else{
			System.err.println("Could not change scoops to " + scoops);
			System.exit(-1);
		}
	}

	/**
	 * Change my order completion status
	 */
	public void setStatus(boolean status) {
		myStatus = status;
	}

	/**
	 * Computes the price of the order
	 * @return cost
	 */
	public double getCost() {
		return myScoops * PRICE_PER_SCOOP;
	}

	/**
	 * @return a copy of myself
	 */
	public IceCreamOrder copy() throws Exception {
		return new IceCreamOrder(myScoops, myFlavor, myStatus);
	}

	/**
	 * Compares one order to another
	 * @param other
	 * @return whether or not the order is bigger than some other order
	 */
	public boolean isBigger(IceCreamOrder other){
		return myScoops > other.getScoops();
	}
}
