package c09classes.lecture.icecream;


/**
 * IceCreamOrder models an order for an ice cream shop
 * 
 * IceCreamOrder invariant
 * (myContainer == 'D' || (myContainer == 'C' && mySprinkles == false)) &&
 * (myScoops >= 1) && (myFlavor != null)
 * Dishes can have sprinkles or not, but cones cannot have sprinkles
 * @author smn4
 * @version Spring 2012
 */
public class IceCreamOrder2 {
	private int myScoops;
	private String myFlavor;
	private char myContainer;
	private boolean mySprinkles;
	
	private static final double PRICE_PER_SCOOP = 1.50;
	
	public IceCreamOrder2(){
		myScoops  = 1;
		myFlavor = "Vanilla";
		myContainer = 'D';
		mySprinkles = false;
	}
	
	public IceCreamOrder2(int scoops, String flavor, char container, boolean sprinkles){
		if (isValidOrder (scoops, flavor, container, sprinkles)){
			myScoops = scoops;
			myFlavor = flavor;
			myContainer = container;
			mySprinkles = sprinkles;
		}
		else {
			System.err.println("Invalid ice cream order: " + scoops + " " + flavor + " "
					+ container + " " + sprinkles);
			System.exit(-1);
		}
		
	}

	private boolean isValidOrder(int scoops, String flavor, char container, boolean sprinkles) {
		if (!(isValidContainer(container, sprinkles))){
			return false;
		}
		
		if (!(isValidScoops(scoops))){
			return false;
		}
		
		if (flavor == null){
			return false;
		}
		return true;
	}
	
	private boolean isValidScoops(int scoops) {
		return scoops >= 1;
	}
	public String toString(){
		String container;
		if (myContainer == 'C'){
			container = "cone";}
		else{
			container = "dish";
		}
		
		String result = myScoops + " scoop(s) of " + myFlavor + " in a " + container;
		if (mySprinkles){
			result += " with sprinkles";
		}
		return result;
	}
	
	public int getScoops(){
		return myScoops;
	}
	
	public String getFlavor(){
		return myFlavor;
	}
	
	public void setScoops(int scoops){
		if (isValidScoops(scoops)){
			myScoops = scoops;
		}
		else{
			System.err.println("Could not change scoops to: " + scoops);
		}
	}
	
	public void setSprinkles(boolean sprinkles){
			
		if (myContainer == 'C' && sprinkles){
			System.err.println("You can't put sprinkles on a cone!");
		}
		else{
			mySprinkles = sprinkles;
		}
	}

	public void setContainer(char container) {
		if(isValidContainer(container, mySprinkles)){
			myContainer = container;
		}
		else{	
			System.err.println("Can't change container to: " + container);
		}	
	}

	private boolean isValidContainer(char container, boolean sprinkles) {
		if (container == 'C' && sprinkles == true){
			return false;
		}
		
		if (container != 'C' && container != 'D'){
			return false;
		}
		return true;
	}

	public double getCost() {
		return myScoops * PRICE_PER_SCOOP;
	}
	
	public boolean isBigger(IceCreamOrder2 other){
		return other.getScoops() < myScoops;
	}
}
