public class Ball {

  public final float BOOST_FACTOR = 1.01;

  private int myRadius;
  private float myX, myY, myVelocityX, myVelocityY, myAccelerationX, myAccelerationY;
  private color myColor;

  public Ball(float x, float y, int radius, float velocityX, float velocityY, float accelerationX, float accelerationY) {
    myX = constrain(x, 0, width);
    myY = constrain(y, 0, MAX_INT);
    myRadius = constrain(radius, 0, MAX_INT);
    myVelocityX = velocityX;
    myVelocityY = velocityY;
    myAccelerationX = accelerationX;
    myAccelerationY = accelerationY;
    myColor = color(255);
  }

  // Constructor for stationary balls
  public Ball(float x, float y, int radius) {
    myX = x;
    myY = y;
    myRadius = radius;
    myVelocityX = myVelocityY = myAccelerationX = myAccelerationY = 0.0;
    myColor = color(random(255), random(255), random(255), random(255));
  }

  public float getX() {
    return myX;
  }
  public float getY() {
    return myY;
  }
  public int getRadius() {
    return myRadius;
  }

  public void render() {
    stroke(0);
    fill(myColor);
    ellipse(myX, myY, myRadius * 2, myRadius * 2);
  }

  public void move(Paddle paddle) {
    myX += myVelocityX;
    myY += myVelocityY;
    if (hitsSide()) {
      myVelocityX *= -1;
      myAccelerationX *= -1;
    }
    if (hitsBottom(paddle) && hitsPaddle(paddle)) {
      myY = height - myRadius - paddle.getHeight();
      myVelocityY *= -1;
    }
    if (abs(myVelocityX) < 0.01) {
      myVelocityX = 0.0;
    }
    myVelocityX += myAccelerationX;
    myVelocityY += myAccelerationY;
  }

  public boolean hitsSide() {
    return (myX - myRadius <= 0) || (myX + myRadius >= width);
  }

  public boolean hitsBottom(Paddle paddle) {
    return abs(myY + myRadius + paddle.getHeight() - height) <= myVelocityY;
  }

  public boolean hitsPaddle(Paddle paddle) {
    return abs(myX - paddle.getX()) < (paddle.getWidth() / 2);
  }

  public boolean hitsTarget(Ball target) {
    return (myRadius + target.getRadius()) >= dist(myX, myY, target.getX(), target.getY());
  }

  // Collision algorithm based on K. Terzidis, Algorithms for Visual Design
  public void collide(Ball target) {
    float angle = atan2(target.getY() - myY, target.getX() - myX);
    // The following simplifications work at slower speeds, but not at full speed. - kvlinden, 30july2009
    //    myVelocityX = (myVelocityX - (cos(angle) * (myRadius + target.getRadius()))) * BOOST_FACTOR;
    //    myVelocityY = (myVelocityY - (sin(angle) * (myRadius + target.getRadius()))) * BOOST_FACTOR;
    float targetX = myX + cos(angle) * (myRadius + target.getRadius());
    float targetY = myY + sin(angle) * (myRadius + target.getRadius());
    float ax = (targetX - target.getX());
    float ay = (targetY - target.getY());
    myVelocityX = (myVelocityX - ax) * BOOST_FACTOR;
    myVelocityY = (myVelocityY - ay) * BOOST_FACTOR;
    //        target.myVelocityX += (ax);
    //        target.myVelocityY += (ay);
  }

}












