package c09classes.text.temperature;

import c10quality.text.temperatureGUI.Temperature2.Scale;

/**
 * This class implements a standard temperature object.
 * 
 * Temperature class invariant: myScale == 'C' && myDegrees >=
 * ABSOLUTE_ZERO_CELSIUS || myScale == 'F' && myDegrees >=
 * ABSOLUTE_ZERO_FAHRENHEIT || myScale == 'K' && myDegrees >=
 * ABSOLUTE_ZERO_KELVIN
 * 
 * 
 * @author kvlinden
 * @version Spring, 2012 (based on the original ANN implementation)
 */
public class Temperature {

	public final static double ABSOLUTE_ZERO_CELSIUS = -273.15, ABSOLUTE_ZERO_FAHRENHEIT = -459.67,
			ABSOLUTE_ZERO_KELVIN = 0.0;

	private double myDegrees;
	private char myScale;

	public Temperature() {
		myDegrees = 0;
		myScale = 'C';
	}

	public Temperature(double degrees, char scale) {
		if (isValidTemperature(degrees, scale)) {
			myDegrees = degrees;
			myScale = Character.toUpperCase(scale);
		} else {
			System.err.println("Invalid Temperature: (" + degrees + "," + scale + ")");
			System.exit(-1);
		}
	}

	public double getDegrees() {
		return myDegrees;
	}

	public char getScale() {
		return myScale;
	}

	public void change(double amount) {
		double newDegrees = myDegrees + amount;
		if (isValidTemperature(newDegrees, myScale)) {
			myDegrees = newDegrees;
		} else {
			System.err.println("Invalid Temperature change: (" + newDegrees + "," + myScale + ")");
			System.exit(-1);
		}
	}

	public boolean lessThan(Temperature otherTemperature) {
		return compareTo(otherTemperature) < 0;
	}

//	public boolean lessThan(Temperature otherTemperature) {
//		Temperature temp = otherTemperature.copy();
//		temp.setScale(myScale);
//		return myDegrees < temp.getDegrees();
//	}

	public boolean lessThanOrEqualTo(Temperature otherTemperature) {
		return compareTo(otherTemperature) <= 0;
	}

	public boolean greaterThan(Temperature otherTemperature) {
		return compareTo(otherTemperature) == 1;
	}

	public boolean greaterThanOrEqualTo(Temperature otherTemperature) {
		return compareTo(otherTemperature) >= 0;
	}

	public boolean equals(Temperature otherTemperature) {
		return compareTo(otherTemperature) == 0;
	}

	private int compareTo(Temperature otherTemperature) {
		Temperature temp = otherTemperature.copy();
		temp.setScale(myScale);
		if (myDegrees < temp.getDegrees()) {
			return -1;
		} else if (myDegrees > temp.getDegrees()) {
			return 1;
		} else {
			return 0;
		}
	}

	public Temperature copy() {
		return new Temperature(myDegrees, myScale);
	}

	public void setScale(char scale) {
		if (Character.toUpperCase(scale) == 'C') {
			convertToCelsius();
		} else if (Character.toUpperCase(scale) == 'F') {
			convertToFahrenheit();
		} else if (Character.toUpperCase(scale) == 'K') {
			convertToKelvin();
		} else {
			System.err.println("Invalid Temperature scale: (" + scale + ")");
			System.exit(-1);
		}

	}

	public void setScaleOLD(Scale scale) throws Exception {
		if (scale == Scale.CELSIUS) {
			convertToCelsius();
		} else if (scale == Scale.FAHRENHEIT) {
			convertToFahrenheit();
		} else if (scale == Scale.KELVIN) {
			convertToKelvin();
		}
	}

	private void convertToCelsius() {
		if (myScale == 'F') {
			myDegrees = 5.0 / 9.0 * (myDegrees - 32.0);
		} else if (myScale == 'K') {
			myDegrees = myDegrees - 273.15;
		}
		myScale = 'C';
	}

	private void convertToFahrenheit() {
		if (myScale == 'C') {
			myDegrees = myDegrees * 1.8 + 32.0;
		} else if (myScale == 'K') {
			myDegrees = (myDegrees - 273.15) * 1.8 + 32.0;
		}
		myScale = 'F';
	}

	private void convertToKelvin() {
		if (myScale == 'C') {
			myDegrees = myDegrees + 273.15;
		} else if (myScale == 'F') {
			myDegrees = 5.0 / 9.0 * (myDegrees - 32.0) + 273.15;
		}
		myScale = 'K';
	}

	// Old convert methods from ANN, removed from the Processing version of the
	// text because they don't make as much sense as the convert mutator
	// implemented above.

	// public Temperature inCelsius() {
	// if (myScale == 'C') {
	// return new Temperature(myDegrees, 'C');
	// } else if (myScale == 'F') {
	// return new Temperature((myDegrees - 32) / 1.8, 'C');
	// } else {
	// return new Temperature(myDegrees - 273.15, 'C');
	// }
	// }
	//
	// public Temperature inFahrenheit() {
	// if (myScale == 'C') {
	// return new Temperature((myDegrees * 1.8) + 32.0, 'F');
	// } else if (myScale == 'F') {
	// return new Temperature(myDegrees, 'F');
	// } else {
	// return new Temperature(((myDegrees - 273.15) * 1.8) + 32.0, 'F');
	// }
	// }
	//
	// public Temperature inKelvin() {
	// if (myScale == 'C') {
	// return new Temperature(myDegrees + 273.15, 'K');
	// } else if (myScale == 'F') {
	// return new Temperature(((myDegrees - 32) / 1.8) + 273.15, 'K');
	// } else {
	// return new Temperature(myDegrees, 'K');
	// }
	// }

	public static boolean isValidTemperature(double degrees, char scale) {
		if (Character.toUpperCase(scale) == 'C') {
			return degrees >= ABSOLUTE_ZERO_CELSIUS;
		} else if (Character.toUpperCase(scale) == 'F') {
			return degrees >= ABSOLUTE_ZERO_FAHRENHEIT;
		} else if (Character.toUpperCase(scale) == 'K') {
			return degrees >= ABSOLUTE_ZERO_KELVIN;
		} else {
			return false;
		}
	}

	/*
	 * switch version public static boolean isValidTemperature(double degrees,
	 * char scale) { switch (scale) { case 'c': case 'C': return degrees >=
	 * ABSOLUTE_ZERO_CELSIUS; case 'f': case 'F': return degrees >=
	 * ABSOLUTE_ZERO_FAHRENHEIT; case 'k': case 'K': return degrees >=
	 * ABSOLUTE_ZERO_KELVIN; default: return false; } }
	 */

	public String toString() {
		return myDegrees + " " + myScale;
	}
}
