package c09classes.text.temperature;

import static c10quality.text.temperatureGUI.Temperature2.Scale.CELSIUS;
import static c10quality.text.temperatureGUI.Temperature2.Scale.FAHRENHEIT;
import static c10quality.text.temperatureGUI.Temperature2.Scale.KELVIN;
import static org.junit.Assert.*;

import org.junit.Test;

import c10quality.text.temperatureGUI.Temperature2;

public class TemperatureTest {

	@Test
	public void defaultConstructorTests() {
		Temperature t1 = new Temperature();
		assertEquals(0.0, t1.getDegrees(), 1e-5);
		assertEquals('C', t1.getScale());
	}

	@Test
	public void explicitValueConstructorTests() {
		Temperature t1 = new Temperature(0, 'K');
		assertEquals(0.0, t1.getDegrees(), 1e-5);
		assertEquals('K', t1.getScale());

		t1 = new Temperature(98.6, 'f');
		assertEquals(98.6, t1.getDegrees(), 1e-5);
		assertEquals('F', t1.getScale());
	}
	
	@Test
	public void equalityTests() {
		Temperature t; 
		
		// Test freezing temperature equivalents.
		t = new Temperature(0.0, 'C');
		t.setScale('F');
		assertEquals(32.0, t.getDegrees(), 1e-5);
		t.setScale('K');
		assertEquals(273.15, t.getDegrees(), 1e-5);

		// Test boiling temperature equivalents.
		t = new Temperature(373.15, 'K');
		t.setScale('F');
		assertEquals(212.0, t.getDegrees(), 1e-5);
		t.setScale('C');
		assertEquals(100.0, t.getDegrees(), 1e-5);

		// Test absolute zero temperature equivalents.
		t = new Temperature(-459.67, 'F');
		t.setScale('K');
		assertEquals(0.0, t.getDegrees(), 1e-5);
		t.setScale('C');
		assertEquals(-273.15, t.getDegrees(), 1e-5);

		// Test large temperature equivalents.
		t = new Temperature(1799540.33, 'F');
		t.setScale('C');
		assertEquals(999726.85, t.getDegrees(), 1e-5);
		t.setScale('K');
		assertEquals(1000000.0, t.getDegrees(), 1e-5);
		
	}	
	
	@Test
	public void comparatorsTest() {
		Temperature2 t1 = new Temperature2(5.0, KELVIN);
		assertTrue(t1.equals(t1));
		assertFalse(t1.lessThan(t1));
		assertTrue(t1.lessThanOrEquals(t1));
		assertFalse(t1.greaterThan(t1));
		assertTrue(t1.greaterThanOrEquals(t1));
		Temperature2 t2 = new Temperature2(25.0, CELSIUS);
		assertFalse(t1.equals(t2));
		assertTrue(t1.lessThan(t2));
		assertFalse(t1.greaterThan(t2));
		assertTrue(t1.lessThanOrEquals(t2));
		assertFalse(t1.greaterThanOrEquals(t2));
		Temperature2 t3 = new Temperature2(5.0, FAHRENHEIT);
		assertFalse(t1.equals(t3));
		assertTrue(t1.lessThan(t3));
		assertTrue(t1.lessThanOrEquals(t3));
		assertFalse(t1.greaterThan(t2));
		assertFalse(t1.greaterThanOrEquals(t3));
	}

	@Test
	public void changeTests() {
		Temperature t1 = new Temperature(50.0, 'F');
		t1.change(10.0);
		assertEquals(60.0, t1.getDegrees(), 0e-5);
		t1.change(-20.0);
		assertEquals(40.0, t1.getDegrees(), 0e-5);
	}
	
	public void testCopy() {
		Temperature2 t1 = new Temperature2(10000.5, CELSIUS);
		Temperature2 t2 = t1.copy();
		assertTrue(t1.equals(t2));
		assertNotSame(t1, t2);
	}

}
