package c10quality.text.temperature;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;

/**
 * Unit tests for the temperature class for Chapter 10 (and 9).
 * 
 * @author kvlinden
 * @version Spring, 2012
 */
public class TemperatureTest {

	@Test
	public void defaultConstructorTests() {
		// Verify that the default temperature is 0.0 Celsius.
		Temperature t1 = new Temperature();
		assertEquals(0.0, t1.getDegrees(), 1e-5);
		assertEquals('C', t1.getScale());
	}
	
	@Test
	public void emptyTest() {
		// Empty test pass by default.
	}

	@Test
	public void explicitValueConstructorTests() {
		// Verify that other temperatures can be constructed and accessed.
		try {
			Temperature t1 = new Temperature(0, 'K');
			assertEquals(0.0, t1.getDegrees(), 1e-5);
			assertEquals('K', t1.getScale());

			t1 = new Temperature(98.6, 'f');
			assertEquals(98.6, t1.getDegrees(), 1e-5);
			assertEquals('F', t1.getScale());
		} catch (Exception e) {
			fail("inappropriate exception raised...");
		}
	}

	@Test
	public void badExplicitValueConstructorTests() {
		// Try some invalid temperatures.
		badExplicitValueConstructorUtility(10.0, 'X');
		badExplicitValueConstructorUtility(-0.01, 'K');
	}

	// This utility tries to construct invalid temperatures using try-catch.
	private void badExplicitValueConstructorUtility(double degrees, char scale) {
		Temperature t;
		try {
			t = new Temperature(degrees, scale);
			fail("Exception should have been thrown on: " + degrees + " " + scale);
		} catch (Exception e) {
			// Do nothing; it should throw this exception.
		}
	}

	@Test
	public void equalityTests() {
		Temperature t;
		try {
			// Test freezing temperature equivalents.
			t = new Temperature(0.0, 'C');
			t.setScale('F');
			assertEquals(32.0, t.getDegrees(), 1e-5);
			t.setScale('K');
			assertEquals(273.15, t.getDegrees(), 1e-5);

			// Test boiling temperature equivalents.
			t = new Temperature(373.15, 'K');
			t.setScale('F');
			assertEquals(212.0, t.getDegrees(), 1e-5);
			t.setScale('C');
			assertEquals(100.0, t.getDegrees(), 1e-5);

			// Test absolute zero temperature equivalents.
			t = new Temperature(-459.67, 'F');
			t.setScale('K');
			assertEquals(0.0, t.getDegrees(), 1e-5);
			t.setScale('C');
			assertEquals(-273.15, t.getDegrees(), 1e-5);

			// Test large temperature equivalents.
			t = new Temperature(1799540.33, 'F');
			t.setScale('C');
			assertEquals(999726.85, t.getDegrees(), 1e-5);
			t.setScale('K');
			assertEquals(1000000.0, t.getDegrees(), 1e-5);
		} catch (Exception e) {
			fail("inappropriate exception raised...");
		}
	}

	@Test
	public void badSetScaleTest() {
		Temperature t = new Temperature();
		try {
			t.setScale(' ');
			fail("Exception should have been thrown...");
		} catch (Exception e) {
			// Do nothing; it should throw this exception.
		}
	}

	@Test
	public void comparatorsTest() {
		Temperature t1;
		try {
			t1 = new Temperature(5.0, 'K');
			assertTrue(t1.equals(t1));
			assertFalse(t1.lessThan(t1));
			assertTrue(t1.lessThanOrEqualTo(t1));
			assertFalse(t1.greaterThan(t1));
			assertTrue(t1.greaterThanOrEqualTo(t1));
			Temperature t2 = new Temperature(25.0, 'C');
			assertFalse(t1.equals(t2));
			assertTrue(t1.lessThan(t2));
			assertFalse(t1.greaterThan(t2));
			assertTrue(t1.lessThanOrEqualTo(t2));
			assertFalse(t1.greaterThanOrEqualTo(t2));
			Temperature t3 = new Temperature(5.0, 'F');
			assertFalse(t1.equals(t3));
			assertTrue(t1.lessThan(t3));
			assertTrue(t1.lessThanOrEqualTo(t3));
			assertFalse(t1.greaterThan(t2));
			assertFalse(t1.greaterThanOrEqualTo(t3));
		} catch (Exception e) {
			fail("inappropriate exception raised...");
		}
	}

	@Test
	public void changeTests() {
		try {
			Temperature t1 = new Temperature(50.0, 'F');
			t1.change(10.0);
			assertEquals(60.0, t1.getDegrees(), 0e-5);
			t1.change(-20.0);
			assertEquals(40.0, t1.getDegrees(), 0e-5);
		} catch (Exception e) {
			fail("inappropriate exception raised...");
		}
	}

	public void testCopy() {
		Temperature t1;
		try {
			t1 = new Temperature(10000.5, 'C');
			Temperature t2 = t1.copy();
			assertTrue(t1.equals(t2));
			assertNotSame(t1, t2);
		} catch (Exception e) {
			fail("inappropriate exception raised...");
		}
	}

}
