package c10quality.text.temperatureGUI;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;

public class Temperature1Test {

	private static final double DOUBLE_EPSILON = 1e-5;

	@Test
	public void doNothingTest() {
		assertTrue(true);
		assertEquals(1, 1);
		assertEquals(3.14159, Math.PI, DOUBLE_EPSILON);
	}

	@Test
	public void defaultConstructorTest() {
		Temperature1 t = new Temperature1();
		assertEquals(0.0, t.getDegrees(), DOUBLE_EPSILON);
		assertEquals("celsius", t.getScale());
	}

	@Test
	public void explicitConstructorTest() {
		try {
			Temperature1 t = new Temperature1(0.0, "celsius");
			assertEquals(0.0, t.getDegrees(), DOUBLE_EPSILON);
			assertEquals("celsius", t.getScale());

			t = new Temperature1(273.15, "kelvin");
			assertEquals(273.15, t.getDegrees(), DOUBLE_EPSILON);
			assertEquals("kelvin", t.getScale());

			t = new Temperature1(32.0, "fahrenheit");
			assertEquals(32.0, t.getDegrees(), DOUBLE_EPSILON);
			assertEquals("fahrenheit", t.getScale());
		} catch (Exception e) {
			fail("Failed explicit constructors test...");
		}

		badTemperatureConstructionUtility(-1.0, "kelvin");
		badTemperatureConstructionUtility(-274.0, "celsius");
		badTemperatureConstructionUtility(-460.0, "fahrenheit");
		badTemperatureConstructionUtility(-460.0, "blob");

		goodTemperatureConstructionUtility(10000.5, "celsius");
		goodTemperatureConstructionUtility(10000.5, "Fahrenheit");
		goodTemperatureConstructionUtility(10000.5, "KELVIN");

	}

	private void goodTemperatureConstructionUtility(double degrees, String scale) {
		try {
			new Temperature1(degrees, scale);
		} catch (Exception e) {
			fail("failed good temperature test...");
		}
	}

	private void badTemperatureConstructionUtility(double degrees, String scale) {
		try {
			new Temperature1(degrees, scale);
			fail("inappropriately constructed a " + degrees + " degree " + scale + " Temperature");
		} catch (Exception e) {
			// It should throw this exception!
		}
	}

	@Test
	public void mutatorGoodTest() {
		try {
			Temperature1 t = new Temperature1();
			t.setScale("fahrenheit");
			assertEquals(32.0, t.getDegrees(), DOUBLE_EPSILON);
			t.setScale("Kelvin");
			assertEquals(273.15, t.getDegrees(), DOUBLE_EPSILON);

			t = new Temperature1(373.15, "kelvin");
			t.setScale("fahrenheit");
			assertEquals(212.0, t.getDegrees(), DOUBLE_EPSILON);
			t.setScale("CELSIUS");
			assertEquals(100.0, t.getDegrees(), DOUBLE_EPSILON);

			t = new Temperature1(-273.15, "celsius");
			t.setScale("fahrenheit");
			assertEquals(-459.67, t.getDegrees(), DOUBLE_EPSILON);
			t.setScale("Kelvin");
			assertEquals(0.0, t.getDegrees(), DOUBLE_EPSILON);

			t = new Temperature1(1799540.33, "fahrenheit");
			t.setScale("Celsius");
			assertEquals(999726.85, t.getDegrees(), DOUBLE_EPSILON);
			t.setScale("kelvin");
			assertEquals(1000000.0, t.getDegrees(), DOUBLE_EPSILON);
		} catch (Exception e) {
			fail("Failed mutator test");
		}
	}

//	@Test
//	public void junkTest() {
//		// This will fail the test.
//		throw new RuntimeException("testing");
//	}

	@Test
	public void mutatorBadTest() {
		Temperature1 t;
		try {
			t = new Temperature1();
			t.setScale("blob");
			fail("Failed mutator test");
		} catch (Exception e) {
			// This should throw an exception.
		}
	}

}
