package c10quality.text.temperatureGUI;

public class Temperature2 {

	private final static double ABSOLUTE_ZERO_CELSIUS = -273.15, ABSOLUTE_ZERO_FAHRENHEIT = -459.67, ABSOLUTE_ZERO_KELVIN = 0.0;

	public static enum Scale {
		FAHRENHEIT, CELSIUS, KELVIN
	}

	private double myDegrees;
	private Scale myScale;

	public Temperature2() {
		myDegrees = 0.0;
		myScale = Scale.CELSIUS;
	}

	public Temperature2(double degrees, Scale scale) {
		if (isValid(degrees, scale)) {
			myDegrees = degrees;
			myScale = scale;
		} else {
			throw new IllegalArgumentException("Invalid temperature: " + degrees + " " + scale);
		}
	}

	private static boolean isValid(double degrees, Scale scale) {
		switch (scale) {
		case CELSIUS:
			return degrees >= ABSOLUTE_ZERO_CELSIUS;
		case FAHRENHEIT:
			return degrees >= ABSOLUTE_ZERO_FAHRENHEIT;
		case KELVIN:
			return degrees >= ABSOLUTE_ZERO_KELVIN;
		default:
			return false;
		}
	}
	
	private static boolean isValidOLD(double degrees, Scale scale) {
		if (scale == Scale.CELSIUS) {
			return degrees >= ABSOLUTE_ZERO_CELSIUS;
		} else if (scale == Scale.FAHRENHEIT) {
			return degrees >= ABSOLUTE_ZERO_FAHRENHEIT;
		} else if (scale == Scale.KELVIN) {
			return degrees >= ABSOLUTE_ZERO_KELVIN;
		} else {
			return false;
		}
	}

	public double getDegrees() {
		return myDegrees;
	}

	public Scale getScale() {
		return myScale;
	}

	public void setScale(Scale scale) {
		switch (scale) {
		case CELSIUS:
			convertToCelsius();
			break;
		case FAHRENHEIT:
			convertToFahrenheit();
			break;
		case KELVIN:
			convertToKelvin();
			break;
		}
	}

	public void setScaleOLD(Scale scale) throws Exception {
		if (scale == Scale.CELSIUS) {
			convertToCelsius();
		} else if (scale == Scale.FAHRENHEIT) {
			convertToFahrenheit();
		} else if (scale == Scale.KELVIN) {
			convertToKelvin();
		}
	}

	private void convertToCelsius() {
		switch (myScale) {
		case FAHRENHEIT:
			myDegrees = 5.0 / 9.0 * (myDegrees - 32.0);
			break;
		case KELVIN:
			myDegrees = myDegrees - 273.15;
			break;
		}
		myScale = Scale.CELSIUS;
	}

	private void convertToFahrenheit() {
		switch (myScale) {
		case CELSIUS:
			myDegrees = myDegrees * 1.8 + 32.0;
			break;
		case KELVIN:
			myDegrees = (myDegrees - 273.15) * 1.8 + 32.0;
			break;
		}
		myScale = Scale.FAHRENHEIT;
	}

	private void convertToKelvin() {
		switch (myScale) {
		case CELSIUS:
			myDegrees = myDegrees + 273.15;
			break;
		case FAHRENHEIT:
			myDegrees = 5.0 / 9.0 * (myDegrees - 32.0) + 273.15;
			break;
		}
		myScale = Scale.KELVIN;
	}

	public int compareTo(Temperature2 otherTemperature) {
		Scale otherTemperatureScale = otherTemperature.getScale();
		otherTemperature.setScale(myScale);
		double otherTemperatureDegreesInMyScale = otherTemperature.getDegrees();
		otherTemperature.setScale(otherTemperatureScale);
		if (myDegrees < otherTemperatureDegreesInMyScale) {
			return -1;
		} else if (myDegrees > otherTemperatureDegreesInMyScale) {
			return 1;
		} else {
			return 0;
		}
	}

	public boolean equals(Temperature2 otherTemperature) {
		return compareTo(otherTemperature) == 0;
	}

	public boolean lessThan(Temperature2 otherTemperature) {
		return compareTo(otherTemperature) < 0;
	}

	public boolean lessThanOrEquals(Temperature2 otherTemperature) {
		return compareTo(otherTemperature) <= 0;
	}

	public boolean greaterThan(Temperature2 otherTemperature) {
		return compareTo(otherTemperature) > 0;
	}

	public boolean greaterThanOrEquals(Temperature2 otherTemperature) {
		return compareTo(otherTemperature) >= 0;
	}

	public Temperature2 copy() {
		return new Temperature2(myDegrees, myScale);
	}
}
