package c10quality.text.temperatureGUI;

import static c10quality.text.temperatureGUI.Temperature2.Scale.*;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

@SuppressWarnings("serial")
public class Temperature2ConverterController extends JFrame implements ActionListener {

	private JTextField celsiusField, fahrenheitField, kelvinField;
	private JTextArea messageField;
	private Temperature2 temperature;
	private Thermometer thermometer;
	private Font font;

	public Temperature2ConverterController() {
		setTitle("Temperature Converter");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		thermometer = new Thermometer();
		thermometer.init();
		thermometer.setPreferredSize(new Dimension(200, 255));
		add(thermometer, BorderLayout.EAST);

		JPanel controller = new JPanel();
		font = new Font("Ariel", 0, 18);
		controller.setLayout(new GridLayout(4, 1));
		celsiusField = new JTextField(13);
		controller.add(createTemperatureControl("Celsius", celsiusField, font));
		fahrenheitField = new JTextField(13);
		controller.add(createTemperatureControl("Fahrenheit", fahrenheitField, font));
		kelvinField = new JTextField(13);
		controller.add(createTemperatureControl("Kelvin", kelvinField, font));
		messageField = createMessageArea();
		controller.add(messageField);
		add(controller, BorderLayout.WEST);
	}

	private JTextArea createMessageArea() {
		JTextArea area = new JTextArea(1, 10);
		area.setLineWrap(true);
		area.setWrapStyleWord(true);
		area.setOpaque(false);
		area.setEditable(false);
		area.setText("Welcome to the temperature converter!");
		area.setFont(font);
		return area;
	}

	private JPanel createTemperatureControl(String name, JTextField field, Font font) {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(1, 2));
		JLabel label = new JLabel(name + ": ", JLabel.RIGHT);
		label.setFont(font);
		panel.add(label);
		field.setFont(font);
		field.setActionCommand(name);
		field.addActionListener(this);
		panel.add(field);
		return panel;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String actionCommand = ae.getActionCommand();
		double degrees;
		messageField.setText("");
		try {
			if (actionCommand.equalsIgnoreCase("celsius")) {
				degrees = Double.parseDouble(celsiusField.getText());
				temperature = new Temperature2(degrees, CELSIUS);
			} else if (actionCommand.equalsIgnoreCase("fahrenheit")) {
				degrees = Double.parseDouble(fahrenheitField.getText());
				temperature = new Temperature2(degrees, FAHRENHEIT);
			} else if (actionCommand.equalsIgnoreCase("kelvin")) {
				degrees = Double.parseDouble(kelvinField.getText());
				temperature = new Temperature2(degrees, KELVIN);
			} 
		} catch (Exception e) {
			messageField.setText("Illegal input: " + e.getMessage());
		}
		if (temperature != null) {
			temperature.setScale(CELSIUS);
			thermometer.setTemperature(temperature.getDegrees());
			celsiusField.setText(new Double(temperature.getDegrees()).toString());
			temperature.setScale(FAHRENHEIT);
			fahrenheitField.setText(new Double(temperature.getDegrees()).toString());
			temperature.setScale(KELVIN);
			kelvinField.setText(new Double(temperature.getDegrees()).toString());
		}
	}

	public static void main(String[] args) {
		Temperature2ConverterController application = new Temperature2ConverterController();
		application.pack();
		application.setVisible(true);
	}

}
