package c10quality.text.temperatureGUI;

import processing.core.PApplet;
import processing.core.PFont;

/**
 * Thermometer implements an animated, three-way thermometer displaying a given
 * temperature on three different scales. It uses animation to change from one
 * temperature to the next.
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
public class Thermometer extends PApplet {

	private final static int BAR_WIDTH = 15, SPACING = 50, SCALE = 2, PADDING = 20,
			CHANGE_QUOTIENT = 10;

	private float temperature, newTemperature; // in Centigrade
	private PFont degreeFont, scaleFont;
	private boolean temperatureSet;

	public Thermometer() {
		temperatureSet = false;
	}

	public void setTemperature(double degreesCelsius) {
		newTemperature = (float) degreesCelsius;
		temperatureSet = true;
	}

	public void setup() {
		size(SPACING * 4, SCALE * 100 + PADDING * 2 + BAR_WIDTH);
		degreeFont = createFont("Calibri", 12, true, "0987654321".toCharArray());
		scaleFont = createFont("CalibriBold", 20, true, "cfk".toCharArray());
		smooth();
	}

	public void draw() {
		background(255);
		if (temperature != newTemperature) {
			temperature += (newTemperature - temperature) / CHANGE_QUOTIENT;
		}
		drawThermometer(50, "Celsius");
		drawThermometer(100, "Fahrenheit");
		drawThermometer(150, "Kelvin");
	}

	void drawThermometer(int xPosition, String scale) {
		float degrees;
		stroke(0);
		fill(255);
		rect(xPosition - BAR_WIDTH / 2.0f, PADDING, BAR_WIDTH, height - PADDING * 2);
		if (temperatureSet && temperature > 100) {
			fill(255, 0, 0);
		}
		arc(xPosition, PADDING + 1, BAR_WIDTH, BAR_WIDTH, PI, TWO_PI);
		if (temperatureSet) {
			fill(255, 0, 0);
			float constrainedTemperature = constrain(temperature, 0, 100);
			noStroke();
			rect(xPosition - BAR_WIDTH / 2.0f, PADDING + (100 - constrainedTemperature) * SCALE,
					BAR_WIDTH, constrainedTemperature * SCALE + BAR_WIDTH);
		}
		stroke(0);
		arc(xPosition, height - PADDING, BAR_WIDTH * 2, BAR_WIDTH * 2, -acos(0.5f), PI + acos(0.5f));
		textAlign(RIGHT);
		fill(0);
		for (int i = 0; i <= 100; i += SCALE * 5) {
			line(xPosition - BAR_WIDTH, PADDING + SCALE * i, xPosition - BAR_WIDTH / 2.0f, PADDING
					+ SCALE * i);
			degrees = 100 - i;
			if (scale.equals("Fahrenheit")) {
				degrees = celsiusToFahrenheit(degrees);
			} else if (scale.equals("Kelvin")) {
				degrees = celsiusToKelvin(degrees);
			}
			textFont(degreeFont);
			text((int) degrees, xPosition - BAR_WIDTH - 1, PADDING + SCALE * i + 5);
		}
		textAlign(CENTER, CENTER);
		textFont(scaleFont);
		text(scale.charAt(0), xPosition, height - PADDING);
	}

	float celsiusToFahrenheit(float celsius) {
		return celsius * 1.8f + 32.0f;
	}

	float celsiusToKelvin(float celsius) {
		return celsius + 273.15f;
	}

}
