final int BAR_WIDTH = 15, SPACING = 50, SCALE = 2, PADDING = 20, CHANGE_QUOTIENT = 10;

float temperature = 1000.0, newTemperature = temperature;  // in Centigrade
PFont degreeFont, scaleFont;

void setup() {
  size(SPACING * 4, SCALE * 100 + PADDING * 2 + BAR_WIDTH);
  degreeFont = loadFont("Calibri-12.vlw");
  scaleFont = loadFont("Calibri-Bold-24.vlw");
  smooth();
}

void draw() {
  background(255);
  if (temperature != newTemperature) {
    temperature += (newTemperature - temperature) / CHANGE_QUOTIENT;
  }
  drawThermometer(50, "Celsius");
  drawThermometer(100, "Fahrenheit");
  drawThermometer(150, "Kelvin");
}

void drawThermometer(int xPosition, String scale) {
  float degrees;
  stroke(0);
  fill(255);
  rect(xPosition - BAR_WIDTH/2.0, PADDING, BAR_WIDTH, height - PADDING * 2);
  if (temperature > 100) {
    fill(255, 0, 0);
  }
  arc(xPosition, PADDING + 1, BAR_WIDTH, BAR_WIDTH, PI, TWO_PI);
  fill(255, 0, 0);
  float constrainedTemperature = constrain(temperature, 0, 100);
  noStroke();
  rect(xPosition - BAR_WIDTH/2.0, PADDING + (100 - constrainedTemperature) * SCALE, BAR_WIDTH, constrainedTemperature * SCALE + BAR_WIDTH);
  stroke(0);
  arc(xPosition, height - PADDING, BAR_WIDTH * 2, BAR_WIDTH * 2, -acos(0.5), PI + acos(0.5));
  textAlign(RIGHT);
  fill(0);
  for (int i = 0; i <= 100; i += SCALE * 5) {
    line(xPosition - BAR_WIDTH, PADDING + SCALE * i, xPosition - BAR_WIDTH/2.0, PADDING  + SCALE * i);
    degrees = 100 - i;
    if (scale.equals("Fahrenheit")) {
      degrees = celsiusToFahrenheit(degrees);
    } 
    else if (scale.equals("Kelvin")) {
      degrees = celsiusToKelvin(degrees);
    }
    textFont(degreeFont);
    text((int)degrees, xPosition - BAR_WIDTH - 1, PADDING + SCALE * i + 5);
  }
  textAlign(CENTER, CENTER);
  textFont(scaleFont);
  text(scale.charAt(0), xPosition, height - PADDING);
}

float celsiusToFahrenheit(float celsius) {
  return celsius * 1.8 + 32.0;
}

float celsiusToKelvin(float celsius) {
  return celsius + 273.15;
}

void mousePressed() {
  newTemperature = 50.0;
}







