package c11ui.lecture.rain;

import processing.core.PApplet;

public class Drop {
	
	  private final int COMBINE_FACTOR = 1;

	  private float myX, myY, myDiameter, myVelocity;

	  /**
	   * Construct a Drop object with default values
	   */
	  public Drop() {
	    myX = 0;
	    myY = 0;
	    myDiameter = 5;
	    myVelocity = 5;
	  }

	  /**
	   * Construct a Drop object
	   * @param x x-coordinate value
	   * @param y y-coordinate value
	   * @param diameter desired diameter (non-negative)
	   * @param velocity desired velocity
	   */
	  public Drop(float x, float y, float diameter, float velocity) {
	    myX = x;
	    myY = y;
	    myDiameter = checkDiameter(diameter);
	    myVelocity = velocity;
	  }

	  /**
	   * Print a message and stop the program if diameter is negative.
	   * @param diameter the value to check
	   * @return verified diameter value
	   */
	  private float checkDiameter(float diameter) {
	    if (diameter < 0) {
	      System.out.println("invalid diameter: " + diameter);
	      System.exit(-1);
	    }
	    return diameter;
	  }

	  /**
	   * @return my x-coordinate value
	   */
	  public float getX() {
	    return myX;
	  }
	  /**
	   * @return my y-coordinate value
	   */
	  public float getY() {
	    return myY;
	  }
	  /**
	   * @return my diameter
	   */
	  public float getDiameter() {
	    return myDiameter;
	  }
	  /**
	   * Set a new diameter value
	   * @param diameter non-negative diameter value
	   */
	  public void setDiameter(float diameter) {
	    myDiameter = checkDiameter(diameter);
	  }
	  /**
	   * @return my velocity value
	   */
	  public float getVelocity() {
	    return myVelocity;
	  }

	  /**
	   * Change my y position based on my drop velocity
	   */
	  public void move() {
	    myY += myVelocity;
	  }

	  /**
	   * Draw myself on the output panel provided that I'm still visible.
	   * @param height the height of the output panel
	   */
	  public void display(PApplet p, int height) {
	    if (myY <= height) {
	      p.fill(p.color(10, 10, 200, 100));
	      p.stroke(0);
	      p.arc(myX, myY, myDiameter, myDiameter, 0, (float)Math.PI);
	      p.noStroke();
	      p.triangle(myX, myY - myDiameter*1.5F, myX - myDiameter/2, myY, myX + myDiameter/2, myY);
	      p.stroke(0);
	      p.line(myX, myY - myDiameter*1.5F, myX - myDiameter/2, myY);
	      p.line(myX, myY - myDiameter*1.5F, myX + myDiameter/2, myY);
	    }
	  }

	  /**
	   * Suck up the volume of neighboring drops if they are close enough.
	   * Dont do this if I am already sucked dry.
	   * @param other reference to one other drop object
	   */
	  public void combine(Drop other) {
	    float otherRadius = other.getDiameter() / 2;
	    if ((myDiameter != 0) && (computeDistance(myX, myY, other.getX(), other.getY()) < (myDiameter/2 + otherRadius + COMBINE_FACTOR))) {
	      myDiameter += Math.sqrt(2) * otherRadius;
	      other.setDiameter(0);
	    }
	  }
	  
	  /**
	   * Compute the Euclidean distance from (x1, y1) to (x2, y2).
	   * (This partially reimplements Processing's dist() method).
	   *
	   * @param x1 the x coordinate of position 1
	   * @param y1 the y coordinate of position 1
	   * @param x2 the x coordinate of position 2
	   * @param y2 the y coordinate of position 2
	   * @return the distance from (x1, y1) to (x2, y2)
	   */
	  double computeDistance(float x1, float y1, float x2, float y2) {
	    float dx = x1-x2;
	    float dy = y1-y2;
	    return Math.sqrt(dx*dx + dy*dy);
	  }


	  /**
	   * @return a copy of myself
	   */
	  public Drop copy() {
	    return new Drop(myX, myY, myDiameter, myVelocity);
	  }


}
