package c11ui.lecture.shaker.shaker5;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * ShakerController is a upgraded version of a Java controller for the
 * Processing raindrops application where the figure drawn is in a separate
 * class (Figure.java).
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
public class ShakerController extends JFrame {

	private ShakerPanel shakerPanel; // for the object version of the panel...
	private JTextField shiftField, outputField;
	private JButton startButton, pauseButton;

	public ShakerController() {
		setTitle("Shaker");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		shakerPanel = new ShakerPanel();
		shakerPanel.init();
		add(shakerPanel, BorderLayout.NORTH);

		// a control panel
		JPanel controlPanel = new JPanel(new FlowLayout());
		startButton = new JButton("Start");
		startButton.setEnabled(false);
		startButton.addActionListener(new StartButtonListener());
		controlPanel.add(startButton);
		pauseButton = new JButton("Pause");
		pauseButton.setEnabled(true);
		pauseButton.addActionListener(new PauseButtonListener());
		controlPanel.add(pauseButton);
		controlPanel.add(new JLabel("Shift factor:"));
		shiftField = new JTextField(3);
		shiftField.setText(String.format("%d", shakerPanel.getShift()));
		controlPanel.add(shiftField);
		shiftField.addActionListener(new ShiftFieldListener());
		add(controlPanel, BorderLayout.CENTER);
		
		outputField = new JTextField();
		outputField.setEditable(false);
		add(outputField, BorderLayout.SOUTH);
	}

	class StartButtonListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			shakerPanel.setRunning(true);
			startButton.setEnabled(false);
			pauseButton.setEnabled(true);
		}
	}

	class PauseButtonListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			shakerPanel.setRunning(false);
			startButton.setEnabled(true);
			pauseButton.setEnabled(false);
		}
	}

	class ShiftFieldListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				shakerPanel.setShift(Integer.parseInt(shiftField.getText()));
			} catch (Exception e) {
				outputField.setText(e.getMessage());
			}
		}
	}

	public static void main(String[] args) {
		ShakerController controller = new ShakerController();
		controller.pack();
		controller.setVisible(true);
	}

}
