package c12lists.lecture.students;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * StudentRetrieverController implements a search tool for student records.
 * 
 * @author kvlinden
 * @version based on original Java text example, Spring, 2012
 */
@SuppressWarnings("serial")
public class StudentRetrieverController extends JFrame {

	private final String FILENAME = "src/c12lists/lecture/students/students.txt";

	private StudentSequence myStudents;
	private JLabel myIDLabel, myCreditsLabel, myGPALabel;
	private JTextField myIDField, myFirstNameField, myLastNameField, myYearField, myCreditsField,
			myGPAField, myStatusField;
	private JPanel inputPanel, outputPanel, namePanel, dataPanel, statusPanel;

	public StudentRetrieverController() {
		setTitle("Student Data Retrival System");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		// The input panel allows the user to enter a student ID number.
		inputPanel = new JPanel();
		myIDLabel = new JLabel("Enter ID Number: ");
		inputPanel.add(myIDLabel);
		myIDField = new JTextField(12);
		myIDField.addActionListener(new IDFieldListener());
		inputPanel.add(myIDField);
		add(inputPanel, BorderLayout.NORTH);

		// The output panel displays information (about the student and about
		// the status of the program).
		outputPanel = new JPanel(new GridLayout(3, 1));

		namePanel = new JPanel(new FlowLayout());
		myFirstNameField = new JTextField(15);
		myFirstNameField.setEditable(false);
		myLastNameField = new JTextField(15);
		myLastNameField.setEditable(false);
		namePanel.add(myFirstNameField);
		namePanel.add(myLastNameField);
		outputPanel.add(namePanel);

		dataPanel = new JPanel(new FlowLayout());
		myYearField = new JTextField(10);
		myYearField.setEditable(false);
		myCreditsLabel = new JLabel("Credits:");
		myCreditsField = new JTextField(6);
		myCreditsField.setEditable(false);
		myGPALabel = new JLabel("GPA:");
		myGPAField = new JTextField(6);
		myGPAField.setEditable(false);
		dataPanel = new JPanel(new FlowLayout());
		dataPanel.add(myYearField);
		dataPanel.add(myCreditsLabel);
		dataPanel.add(myCreditsField);
		dataPanel.add(myGPALabel);
		dataPanel.add(myGPAField);
		outputPanel.add(dataPanel);

		statusPanel = new JPanel(new FlowLayout());
		myStatusField = new JTextField(31);
		myStatusField.setEditable(false);
		statusPanel.add(myStatusField);
		outputPanel.add(statusPanel);

		add(outputPanel, BorderLayout.SOUTH);

		// myStudents maintains a list of students read from the given file.
		try {
			myStudents = new StudentSequence(FILENAME);
		} catch (Exception e) {
			enterErrorState(e.getMessage());
		}
	}

	// Display the information for the given student (assumed to be valid) and
	// clear the message panel.
	public void enterGoodIDState(Student stu) {
		myFirstNameField.setText(stu.getFirstName());
		myLastNameField.setText(stu.getLastName());
		myYearField.setText(stu.getYear());
		myCreditsField.setText(Double.toString(stu.getCredits()));
		myGPAField.setText(Double.toString(stu.getGPA()));
		myStatusField.setText("");
	}

	// Clear the output panel and display the given error message.
	public void enterErrorState(String statusMsg) {
		myFirstNameField.setText("");
		myLastNameField.setText("");
		myYearField.setText("");
		myCreditsField.setText("");
		myGPAField.setText("");
		myStatusField.setText(statusMsg);
	}

	// Handle the case in which the user has entered an ID number.
	class IDFieldListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				int id = Integer.parseInt(myIDField.getText());
				Student student = myStudents.getStudent(id);
				if (student != null) {
					enterGoodIDState(student);
				} else {
					enterErrorState("No student found with that ID");
				}
			} catch (Exception e) {
				enterErrorState("Bad input, please try again.");
			}
		}
	}

	public static void main(String[] args) {
		StudentRetrieverController application = new StudentRetrieverController();
		application.pack();
		application.setVisible(true);
	}
}
