package c12lists.lecture.students;

import java.io.File;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * StudentSequence models a sequence of student objects.
 * 
 * @author kvlinden
 * @version based on original text implementation, Spring, 2012
 * @see Student
 */
class StudentSequence {

	private ArrayList<Student> myStudents;

	/**
	 * THis constructor reads the list of students from the file with the given
	 * filename. The file is assumed to have the appropriate format. Any
	 * file-handling exceptions are thrown out to the calling program.
	 * 
	 * This method reads the lines of the file, passing each one to the Student
	 * object constructor to be parsed.
	 * 
	 * @param fileName
	 *            the file from which to read the student records
	 * @throws Exception
	 */
	public StudentSequence(String fileName) throws Exception {
		myStudents = new ArrayList<Student>();
		Scanner fileIn = new Scanner(new File(fileName));
		while (fileIn.hasNextLine()) {
			myStudents.add(new Student(fileIn.nextLine()));
		}
		fileIn.close();
	}

	/**
	 * Finds and returns the student with the given ID
	 * 
	 * @param id
	 *            the ID of the student to return
	 * @return the requested student or null if no such student is found
	 */
	public Student getStudent(int id) throws Exception {
		for (int i = 0; i < myStudents.size(); i++) {
			if (myStudents.get(i).getIdNumber() == id)
				return myStudents.get(i);
		}
		return null;
	}

	/**
	 * Return the current number of students.
	 */
	public int length() {
		return myStudents.size();
	}
}
