package c12lists.text.characterdrill.characterdrill2;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URISyntaxException;

import javax.swing.*;

/**
 * ChineseDrillController implements a simple Chinese character review
 * application. This version adds file handling and image files, but drops
 * support for Unicode character strings (because they are harder to read from
 * simple text files).
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
@SuppressWarnings("serial")
public class CharacterDrillController extends JFrame {

	private final static String DATA_PATH = "src/c12lists/text/characterdrill/characterdrill2/data/";

	private CharacterDrillPanel drillPanel;
	private JLabel hintLabel;
	private JTextField guessField;
	private JButton giveupButton;
	private CharacterDrill drill;

	public CharacterDrillController() throws Exception {
		setTitle("Character Character Drill");
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		Font font = new Font("Arial Unicode MS", 0, 24);

		//drill = new CharacterDrill(DATA_PATH);
		drill = new CharacterDrill(DATA_PATH, "characters.txt");

		JPanel guessController = new JPanel();
		guessController.setLayout(new FlowLayout());

		JLabel guessLabel = new JLabel("English: ");
		guessLabel.setFont(font);
		guessController.add(guessLabel);

		guessField = new JTextField(15);
		guessField.setFont(font);
		guessField.setActionCommand("Guess");
		guessField.addActionListener(new GuessFieldListener());
		guessController.add(guessField);

		giveupButton = new JButton("Give Up");
		giveupButton.setFont(font);
		giveupButton.addActionListener(new GiveupButtonListener());
		guessController.add(giveupButton);

		add(guessController, BorderLayout.SOUTH);

		hintLabel = new JLabel();
		hintLabel.setFont(font);
		hintLabel.setText(drill.getHintText());
		add(hintLabel, BorderLayout.CENTER);

		drillPanel = new CharacterDrillPanel();
		drillPanel.init();
		add(drillPanel, BorderLayout.NORTH);
		drillPanel.setImage(drill.getHintImageFilename());

	}

	class GuessFieldListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				if (drill.guess(guessField.getText())) {
					drill.reset();
					hintLabel.setText("Right! " + drill.getHintText());
					drillPanel.setImage(drill.getHintImageFilename());
				} else {
					hintLabel.setText("Guess again - " + drill.getHintText());
				}
				guessField.setText("");
			} catch (Exception e) {

			}
		}
	}

	class GiveupButtonListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				String answer = drill.getAnswer();
				drill.reset();
				hintLabel.setText("Answer: " + answer + " - Next: " + drill.getHintText());
				drillPanel.setImage(drill.getHintImageFilename());
				guessField.setText("");
			} catch (Exception e) {
				hintLabel.setText(e.getMessage());
			}
		}
	}

	public static void main(String[] args) throws Exception {
		CharacterDrillController application = new CharacterDrillController();
		application.pack();
		application.setVisible(true);
	}

}
