package c12lists.text.characterdrill.characterdrillOld;

import java.util.Scanner;

/**
 * Character models a single Chinese character, with English translation,
 * image/audio filename, PinYin transcription and sample sentence (in Unicode).
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
public class Character {

	private String myEnglishTranslation, myPinyin, myImageFilename, myPronunciationFilename,
			mySentence;

	public Character(String englishTranslation, String pinyin, String imageFilename,
			String pronunciationFilename) {
		myEnglishTranslation = englishTranslation;
		myPinyin = pinyin;
		myImageFilename = imageFilename;
		myPronunciationFilename = pronunciationFilename;
	}

	public Character(String line) throws Exception {
		Scanner scanner = new Scanner(line);
		myEnglishTranslation = scanner.next();
		myPinyin = scanner.next();
		myImageFilename = scanner.next();
		myPronunciationFilename = scanner.next();
		mySentence = "";
		while (scanner.hasNext()) {
			mySentence += encodeChinese(scanner.next());
		}
	}

	public String getEnglishTranslation() {
		return myEnglishTranslation;
	}

	public String getPinyin() {
		return myPinyin;
	}

	public String getImageFilename() {
		return myImageFilename;
	}

	public String getPronunciationFilename() {
		return myPronunciationFilename;
	}

	public String getSentence() {
		if (mySentence.equals("")) {
			return "no sentence available...";
		} else {
			return mySentence;
		}
	}

	// Converts from pinyin to a UNICODE character - Encodings determined using
	// http://www.mandarintools.com/chardict_u8.html
	//
	// TODO: There should be a way to read UNICODE characters directly from the
	// file.
	private static String encodeChinese(String character) throws Exception {
		if (character.equals("wo3")) {
			return "\u6211";
		} else if (character.equals("ni3")) {
			return "\u4F60";
		} else if (character.equals("ai4")) {
			return "\u611B";
		} else if (character.equals("ta1he")) {
			return "\u4ED6";
		} else if (character.equals("ta1she")) {
			return "\u5979";
		} else if (character.equals("ta1it")) {
			return "\u5B83";
		} else if (character.equals("men")) {
			return "\u5011";
		} else if (character.equals("hao3")) {
			return "\u597D";
		} else {
			throw new Exception("unrecognized character code: " + character);
		}
	}

}
