package c12lists.text.characterdrill.characterdrillOld;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URISyntaxException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.*;

/**
 * ChineseDrillController implements a simple Chinese character review
 * application.
 * 
 * The CharacterDrillPanel requires Minim; find the *.jar files for this package
 * in the standard course repository. It also requires that appropriately
 * configured data, image and audio files be stored in the data sub-directory.
 * 
 * @author kvlinden
 * @version Fall, 2009
 */
@SuppressWarnings("serial")
public class CharacterDrillController extends JFrame implements ActionListener {

	private final static String DATA_PATH = "src/c11files/characterdrill/data/";

	private CharacterDrillPanel drillPanel;
	private JLabel hintLabel;
	private JTextField guessField;
	private JButton giveupButton;
	private CharacterDrill2 drill;

	public CharacterDrillController() throws Exception {
		setTitle("Character Character Drill");
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		Font font = new Font("Arial Unicode MS", 0, 24);

		// The hard-coded Character objects...
		// drill = new CharacterDrill0();

		// Character objects loaded from the hard-coded filename (relative to
		// the root eclipse path)...
		// drill = new CharacterDrill1(DATA_PATH + "characters1.txt");
		//drill = new CharacterDrill2(DATA_PATH, "characters2.txt");

		// Character objects loaded from a user-specified filename...
		 JFileChooser fileChooser = new JFileChooser();
		 fileChooser.setDialogTitle("Choose an input filename");
		 fileChooser.showSaveDialog(this);
		 drill = new CharacterDrill2(fileChooser.getSelectedFile().getParent(), fileChooser.getSelectedFile().getName());

		hintLabel = new JLabel();
		hintLabel.setFont(font);
		add(hintLabel, BorderLayout.CENTER);

		JPanel guessController = new JPanel();
		guessController.setLayout(new FlowLayout());
		JLabel guessLabel = new JLabel("PinYin: ");
		guessLabel.setFont(font);
		guessController.add(guessLabel);
		guessField = new JTextField(15);
		guessField.setFont(font);
		guessField.setActionCommand("Guess");
		guessField.addActionListener(this);
		guessController.add(guessField);
		giveupButton = new JButton("Give Up");
		giveupButton.setFont(font);
		giveupButton.addActionListener(this);
		guessController.add(giveupButton);
		add(guessController, BorderLayout.SOUTH);

		hintLabel.setText(drill.getHintText());
		drillPanel = new CharacterDrillPanel();
		drillPanel.init();
		add(drillPanel, BorderLayout.NORTH);
		initializeDrill();
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String actionCommand = event.getActionCommand();
		try {
			if (actionCommand.equalsIgnoreCase("Guess")) {
				if (drill.guess(guessField.getText())) {
					drill.reset();
					hintLabel.setText("Right! " + drill.getHintText());
					initializeDrill();
				} else {
					hintLabel.setText("Guess again - " + drill.getHintText());
				}
			} else if (actionCommand.equalsIgnoreCase("Give Up")) {
				String answer = drill.getAnswer();
				drill.reset();
				hintLabel.setText("Answer: " + answer + " - Next: " + drill.getHintText());
				initializeDrill();
			}
			guessField.setText("");
		} catch (URISyntaxException e) {
			// This should never happen...
			e.printStackTrace();
		}
	}

	private void initializeDrill() throws URISyntaxException {
	    drillPanel.setImage(drill.getHintImageFilename());
		drillPanel.setPronunciation(drill.getHintPronunciationFilename());
		drillPanel.playPronunciation();
	}

	public static void main(String[] args) throws Exception {
		CharacterDrillController application = new CharacterDrillController();
		application.pack();
		application.setVisible(true);
	}

}
